set props(.movelist.columns) 9
set props(.movelist.rows)    7
set props(.movelist.frames)  1
set props(.movelist.texts)   0
set props(.movelist.components)   "ct up cc g e listbox "
set props(.movelist.count)   6
set props(.movelist.props.ct) "-in .movelist.widgets -column 3 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.movelist.props.up) "-in .movelist.widgets -column 11 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.movelist.props.cc) "-in .movelist.widgets -column 15 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.movelist.props.g) "-in .movelist.widgets -column 3 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.movelist.props.e) "-in .movelist.widgets -column 7 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.movelist.props.listbox) "-in .movelist.widgets -column 1 -row 1 -columnspan 17 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.movelist.frame0) "-in .movelist.widgets -column 1 -row 9 -columnspan 17 -rowspan 7 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.movelist.relief.frame0) groove
