set props(.hotlist.columns) 9
set props(.hotlist.rows)    7
set props(.hotlist.frames)  2
set props(.hotlist.texts)   1
set props(.hotlist.components)   "the-lab ct ad up cc g e r move listbox "
set props(.hotlist.count)   10
set props(.hotlist.props.the-lab) "-in .hotlist.widgets -column 5 -row 5 -columnspan 11 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist.props.ct) "-in .hotlist.widgets -column 3 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.hotlist.props.ad) "-in .hotlist.widgets -column 7 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.hotlist.props.up) "-in .hotlist.widgets -column 11 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.hotlist.props.cc) "-in .hotlist.widgets -column 15 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.hotlist.props.g) "-in .hotlist.widgets -column 3 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.hotlist.props.e) "-in .hotlist.widgets -column 7 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.hotlist.props.r) "-in .hotlist.widgets -column 11 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.hotlist.props.move) "-in .hotlist.widgets -column 15 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.hotlist.props.listbox) "-in .hotlist.widgets -column 1 -row 1 -columnspan 17 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist.frame0) "-in .hotlist.widgets -column 1 -row 3 -columnspan 17 -rowspan 5 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.hotlist.relief.frame0) groove
set props(.hotlist.frame1) "-in .hotlist.widgets -column 1 -row 9 -columnspan 17 -rowspan 7 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.hotlist.relief.frame1) groove
set props(.hotlist.text0) "-in .hotlist.widgets -column 3 -row 5 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.hotlist.text.text0) "Directory path:"
