set props(.option.columns) 9
set props(.option.rows)    13
set props(.option.frames)  3
set props(.option.texts)   0
set props(.option.components)   "show-hidden mark-moves drop-menus mix-files fast-reload label-pause pause-radio label-other verbose shell-patt auto-save auto-menus view-int completion rotating lynx achown cd-follow safe-del button-ok button-save button-cancel label-panel show-backup "
set props(.option.count)   24
set props(.option.props.show-hidden) "-in .option.widgets -column 3 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.mark-moves) "-in .option.widgets -column 3 -row 9 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.drop-menus) "-in .option.widgets -column 3 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.mix-files) "-in .option.widgets -column 3 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.fast-reload) "-in .option.widgets -column 3 -row 15 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.label-pause) "-in .option.widgets -column 3 -row 19 -columnspan 1 -rowspan 3 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.pause-radio) "-in .option.widgets -column 3 -row 23 -columnspan 1 -rowspan 3 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.label-other) "-in .option.widgets -column 11 -row 1 -columnspan 1 -rowspan 3 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky sw"
set props(.option.props.verbose) "-in .option.widgets -column 11 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.shell-patt) "-in .option.widgets -column 11 -row 7 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.auto-save) "-in .option.widgets -column 11 -row 9 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.auto-menus) "-in .option.widgets -column 11 -row 11 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.view-int) "-in .option.widgets -column 11 -row 13 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.completion) "-in .option.widgets -column 11 -row 15 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.rotating) "-in .option.widgets -column 15 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.lynx) "-in .option.widgets -column 15 -row 7 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.achown) "-in .option.widgets -column 15 -row 9 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.cd-follow) "-in .option.widgets -column 15 -row 11 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.safe-del) "-in .option.widgets -column 15 -row 13 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.props.button-ok) "-in .option.widgets -column 13 -row 25 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.option.props.button-save) "-in .option.widgets -column 15 -row 25 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.option.props.button-cancel) "-in .option.widgets -column 17 -row 25 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.option.props.label-panel) "-in .option.widgets -column 3 -row 1 -columnspan 1 -rowspan 3 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky sw"
set props(.option.props.show-backup) "-in .option.widgets -column 3 -row 5 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.option.frame0) "-in .option.widgets -column 9 -row 3 -columnspan 11 -rowspan 15 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.option.relief.frame0) ridge
set props(.option.frame1) "-in .option.widgets -column 1 -row 3 -columnspan 7 -rowspan 15 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.option.relief.frame1) ridge
set props(.option.frame2) "-in .option.widgets -column 1 -row 21 -columnspan 7 -rowspan 7 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.option.relief.frame2) ridge
