set props(.quick_file_mask_copy.columns) 7
set props(.quick_file_mask_copy.rows)    6
set props(.quick_file_mask_copy.frames)  0
set props(.quick_file_mask_copy.texts)   0
set props(.quick_file_mask_copy.components)   "input-def using-shell to follow dive preserve ok back cancel ql input2 stab-sym "
set props(.quick_file_mask_copy.count)   12
set props(.quick_file_mask_copy.props.input-def) "-in .quick_file_mask_copy.widgets -column 3 -row 3 -columnspan 7 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.quick_file_mask_copy.props.using-shell) "-in .quick_file_mask_copy.widgets -column 11 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_file_mask_copy.props.to) "-in .quick_file_mask_copy.widgets -column 1 -row 5 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_file_mask_copy.props.follow) "-in .quick_file_mask_copy.widgets -column 3 -row 9 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_file_mask_copy.props.dive) "-in .quick_file_mask_copy.widgets -column 3 -row 10 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_file_mask_copy.props.preserve) "-in .quick_file_mask_copy.widgets -column 9 -row 9 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_file_mask_copy.props.stab-sym) "-in .quick_file_mask_copy.widgets -column 9 -row 10 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_file_mask_copy.props.ok) "-in .quick_file_mask_copy.widgets -column 3 -row 14 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_file_mask_copy.props.back) "-in .quick_file_mask_copy.widgets -column 7 -row 14 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_file_mask_copy.props.cancel) "-in .quick_file_mask_copy.widgets -column 11 -row 14 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_file_mask_copy.props.ql) "-in .quick_file_mask_copy.widgets -column 1 -row 1 -columnspan 7 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_file_mask_copy.props.input2) "-in .quick_file_mask_copy.widgets -column 3 -row 7 -columnspan 7 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
