set props(.listbox.columns) 4
set props(.listbox.rows)    4
set props(.listbox.frames)  0
set props(.listbox.texts)   0
set props(.listbox.components)   "c li "
set props(.listbox.count)   2
set props(.listbox.props.c) "-in .listbox.widgets -column 3 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.listbox.props.li) "-in .listbox.widgets -column 3 -row 1 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
