set props(.dbits.columns) 4
set props(.dbits.rows)    4
set props(.dbits.frames)  1
set props(.dbits.texts)   0
set props(.dbits.components)   "c o r "
set props(.dbits.count)   3
set props(.dbits.props.c) "-in .dbits.widgets -column 9 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.dbits.props.o) "-in .dbits.widgets -column 9 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.dbits.props.r) "-in .dbits.widgets -column 3 -row 3 -columnspan 1 -rowspan 5 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.dbits.frame0) "-in .dbits.widgets -column 1 -row 1 -columnspan 5 -rowspan 9 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.dbits.relief.frame0) groove
