set props(.panelize.columns) 7
set props(.panelize.rows)    7
set props(.panelize.frames)  0
set props(.panelize.texts)   0
set props(.panelize.components)   "label-command in p r a c li "
set props(.panelize.count)   7
set props(.panelize.props.label-command) "-in .panelize.widgets -column 3 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.panelize.props.in) "-in .panelize.widgets -column 5 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.panelize.props.p) "-in .panelize.widgets -column 3 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.panelize.props.r) "-in .panelize.widgets -column 5 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.panelize.props.a) "-in .panelize.widgets -column 7 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.panelize.props.c) "-in .panelize.widgets -column 11 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.panelize.props.li) "-in .panelize.widgets -column 3 -row 3 -columnspan 9 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
