set props(.sort.columns) 5
set props(.sort.rows)    6
set props(.sort.frames)  1
set props(.sort.texts)   0
set props(.sort.components)   "reverse-check case-check ok-button cancel-button radio-1 "
set props(.sort.count)   5
set props(.sort.props.reverse-check) "-in .sort.widgets -column 7 -row 5 -columnspan 5 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky nw"
set props(.sort.props.case-check) "-in .sort.widgets -column 7 -row 3 -columnspan 5 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky sw"
set props(.sort.props.ok-button) "-in .sort.widgets -column 9 -row 9 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.sort.props.cancel-button) "-in .sort.widgets -column 9 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.sort.props.radio-1) "-in .sort.widgets -column 3 -row 3 -columnspan 1 -rowspan 9 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.sort.frame0) "-in .sort.widgets -column 1 -row 1 -columnspan 5 -rowspan 13 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.sort.relief.frame0) groove
