/* Automatically generated by po2tbl.sed from mc.pot.  */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "libgettext.h"

const struct _msg_ent _msg_tbl[] = {
  {"", 1},
  {"Password:", 2},
  {" Error ", 3},
  {" Fail trying to open file for reading: ", 4},
  {" Cannot get size/permissions info on file: ", 5},
  {" Not an ordinary file: ", 6},
  {" File is too large: ", 7},
  {"\
 \n\
 Increase edit.h:MAXBUF and recompile the editor. ", 8},
  {" Error allocating memory ", 9},
  {" Emacs key: ", 10},
  {" Execute Macro ", 11},
  {" Press macro hotkey: ", 12},
  {" Insert Literal ", 13},
  {" Press any key: ", 14},
  {" Save As ", 15},
  {" Warning ", 16},
  {" A file already exists with this name. ", 17},
  {"Overwrite", 18},
  {"Cancel", 19},
  {" Save as ", 20},
  {" Error trying to save file. ", 21},
  {" Delete macro ", 22},
  {" Error trying to open temp file ", 23},
  {" Error trying to open macro file ", 24},
  {"key '%d 0': ", 25},
  {" Error trying to overwrite macro file ", 26},
  {" Macro ", 27},
  {" Press the macro's new hotkey: ", 28},
  {" Save macro ", 29},
  {" Delete Macro ", 30},
  {" Load macro ", 31},
  {" Confirm save file? : ", 32},
  {" Save file ", 33},
  {"Save", 34},
  {"\
 Current text was modified without a file save. \n\
 Continue discards these changes. ", 35},
  {"Continue", 36},
  {" Load ", 37},
  {" Error trying to open file for reading ", 38},
  {" Block is large, you may not be able to undo this action. ", 39},
  {" Continue ", 40},
  {" Cancel ", 41},
  {" Replace with: ", 42},
  {" Replace ", 43},
  {" Search ", 44},
  {" Enter search text : ", 45},
  {" Enter replace text : ", 46},
  {" Enter argument order : ", 47},
  {"Enter the order of replacement of your scanf format specifiers", 48},
  {" Whole words only ", 49},
  {" Case sensitive ", 50},
  {" Regular expression ", 51},
  {"See the regex man page for how to compose a regular expression", 52},
  {" Backwards ", 53},
  {"Warning: Searching backward can be slow", 54},
  {" Prompt on replace ", 55},
  {"Ask before making each replacement", 56},
  {" Replace all ", 57},
  {" Scanf expression ", 58},
  {"Allows entering of a C format string, see the scanf man page", 59},
  {"Begin search, Enter", 60},
  {"Abort this dialog, Esc", 61},
  {"Replace", 62},
  {"Skip", 63},
  {"Replace all", 64},
  {" Invalid regular expression, or scanf expression with to many conversions ", 65},
  {" Error in replacement format string. ", 66},
  {" %ld replacements made. ", 67},
  {" Search string not found. ", 68},
  {" Quit ", 69},
  {" File was modified, Save with exit? ", 70},
  {"Cancel quit", 71},
  {"&Yes", 72},
  {"&No", 73},
  {"\
 Current text was modified without a file save. \n\
 Save with exit? ", 74},
  {" &Cancel quit ", 75},
  {" &Yes ", 76},
  {" &No ", 77},
  {" Copy to clipboard ", 78},
  {" Unable to save to file. ", 79},
  {" Cut to clipboard ", 80},
  {" Goto line ", 81},
  {" Enter line: ", 82},
  {" Save Block ", 83},
  {" Insert File ", 84},
  {" Insert file ", 85},
  {" Error trying to insert file. ", 86},
  {" Sort block ", 87},
  {" You must first highlight a block of text. ", 88},
  {" Run Sort ", 89},
  {" Enter sort options (see manpage) separated by whitespace: ", 90},
  {" Sort ", 91},
  {" Error trying to execute sort command ", 92},
  {" Sort returned non-zero: ", 93},
  {" Process block ", 94},
  {" Error trying to stat file ", 95},
  {" Mail ", 96},
  {" Word wrap ", 97},
  {" Enter line length, 0 for off: ", 98},
  {"&Open/load...     C-o", 99},
  {"&New              C-n", 100},
  {"&Save              F2", 101},
  {"save &As...       F12", 102},
  {"&Insert file...   F15", 103},
  {"copy to &File...  C-f", 104},
  {"a&Bout...            ", 105},
  {"&Quit             F10", 106},
  {"&New            C-x k", 107},
  {"copy to &File...     ", 108},
  {"&Toggle Mark       F3", 109},
  {"toggle &Ins/overw Ins", 110},
  {"&Copy              F5", 111},
  {"&Move              F6", 112},
  {"&Delete            F8", 113},
  {"&Undo             C-u", 114},
  {"&Beginning     C-PgUp", 115},
  {"&End           C-PgDn", 116},
  {"&Search...         F7", 117},
  {"search &Again     F17", 118},
  {"&Replace...        F4", 119},
  {"&Goto line...            M-l", 120},
  {"insert &Literal...       C-q", 121},
  {"&Refresh screen          C-l", 122},
  {"&Start record macro      C-r", 123},
  {"&Finish record macro...  C-r", 124},
  {"&Execute macro...   C-a, KEY", 125},
  {"delete macr&O...            ", 126},
  {"insert &Date/time           ", 127},
  {"format p&Aragraph        M-p", 128},
  {"'ispell' s&Pell check    C-p", 129},
  {"sor&T...                 M-t", 130},
  {"'indent' &C Formatter    F19", 131},
  {"&Mail...                    ", 132},
  {"&Execute macro... C-x e, KEY", 133},
  {"'ispell' s&Pell check    M-$", 134},
  {"&General...  ", 135},
  {"&Save mode...", 136},
  {"&Layout...", 137},
  {" File ", 138},
  {" Edit ", 139},
  {" Sear/Repl ", 140},
  {" Command ", 141},
  {" Options ", 142},
  {"Open...\tC-o", 143},
  {"New\tC-n", 144},
  {"Save\tF2", 145},
  {"Save as...\tF12", 146},
  {"Insert file...\tF15", 147},
  {"Copy to file...\tC-f", 148},
  {"Disk operations", 149},
  {"Toggle mark\tF3", 150},
  {"Toggle insert/overwrite\tIns", 151},
  {"Copy block to cursor\tF5", 152},
  {"Move block to cursor\tF6", 153},
  {"Delete block\tF8/C-Del", 154},
  {"Copy block to clipbrd\tC-Ins", 155},
  {"Cut block to clipbrd\tS-Del", 156},
  {"Paste block from clipbrd\tS-Ins", 157},
  {"Selection history\tM-Ins", 158},
  {"Undo\tC-BackSpace", 159},
  {"Manipulating blocks of text", 160},
  {" Srch/Replce ", 161},
  {"Search...\tF7", 162},
  {"Search again\tF17", 163},
  {"Replace...\tF4", 164},
  {"Replace again\tF14", 165},
  {"Search for and replace text", 166},
  {"Goto line...\tM-l", 167},
  {"Start record macro\tC-r", 168},
  {"Finish record macro...\tC-r", 169},
  {"Execute macro...\tC-a, KEY", 170},
  {"Delete macro...\t", 171},
  {"Insert date/time\tC-d", 172},
  {"Format paragraph\tM-p", 173},
  {"Macros and internal commands", 174},
  {"None", 175},
  {"Dynamic paragraphing", 176},
  {"Type writer wrap", 177},
  {"Wrap mode", 178},
  {"Key emulation", 179},
  {"Error initialising editor.\n", 180},
  {"Help", 181},
  {"Mark", 182},
  {"Replac", 183},
  {"Copy", 184},
  {"Move", 185},
  {"Search", 186},
  {"Delete", 187},
  {"PullDn", 188},
  {"Quit", 189},
  {"Usage: fixhlp <width> <tocname>\n", 190},
  {"fixhlp: Cannot open toc for writing", 191},
  {"\
\004[Contents]\n\
 Topics:\n\
\n", 192},
  {"&Cancel", 193},
  {"&Set", 194},
  {"S&kip", 195},
  {"Set &all", 196},
  {"owner", 197},
  {"group", 198},
  {"other", 199},
  {"On", 200},
  {"Flag", 201},
  {"Mode", 202},
  {"%6d of %d", 203},
  {" Chown advanced command ", 204},
  {"\
 Couldn't chmod \"%s\" \n\
 %s ", 205},
  {"\
 Couldn't chown \"%s\" \n\
 %s ", 206},
  {" Oops... ", 207},
  {" I can't run the Advanced Chown command on an extfs ", 208},
  {" I can't run the Advanced Chown command on a tarfs ", 209},
  {"Background process:", 210},
  {" Background process error ", 211},
  {" Child died unexpectedly ", 212},
  {" Unknown error in child ", 213},
  {" Background protocol error ", 214},
  {"\
 Background process sent us a request for more arguments \n\
 than we can handle. \n", 215},
  {" Listing mode ", 216},
  {"&Full file list", 217},
  {"&Brief file list", 218},
  {"&Long file list", 219},
  {"&User defined:", 220},
  {"&Icon view", 221},
  {"user &Mini status", 222},
  {"&Ok", 223},
  {"Listing mode", 224},
  {"&Reverse", 225},
  {"case sensi&tive", 226},
  {"Sort order", 227},
  {" confirm &Exit ", 228},
  {" confirm e&Xecute ", 229},
  {" confirm o&Verwrite ", 230},
  {" confirm &Delete ", 231},
  {" Confirmation ", 232},
  {"Full 8 bits output", 233},
  {"ISO 8859-1", 234},
  {"7 bits", 235},
  {"F&ull 8 bits input", 236},
  {" Display bits ", 237},
  {"Always to memory", 238},
  {"If size less than:", 239},
  {"&Always use ftp proxy", 240},
  {"sec", 241},
  {"ftpfs directory cache timeout:", 242},
  {"ftp anonymous password:", 243},
  {"Gzipped tar archive extract:", 244},
  {"Timeout for freeing VFSs:", 245},
  {" Virtual File System Setting ", 246},
  {"Quick cd", 247},
  {"cd", 248},
  {"Symbolic link filename:", 249},
  {"Existing filename (filename symlink will point to):", 250},
  {"Running ", 251},
  {"Stopped", 252},
  {"&Stop", 253},
  {"&Resume", 254},
  {"&Kill", 255},
  {"Background Jobs", 256},
  {"execute/search by others", 257},
  {"write by others", 258},
  {"read by others", 259},
  {"execute/search by group", 260},
  {"write by group", 261},
  {"read by group", 262},
  {"execute/search by owner", 263},
  {"write by owner", 264},
  {"read by owner", 265},
  {"sticky bit", 266},
  {"set group ID on execution", 267},
  {"set user ID on execution", 268},
  {"C&lear marked", 269},
  {"S&et marked", 270},
  {"&Marked all", 271},
  {"Name", 272},
  {"Permissions (Octal)", 273},
  {"Owner name", 274},
  {"Group name", 275},
  {" Chmod command ", 276},
  {" Permission ", 277},
  {"Use SPACE to change", 278},
  {"an option, ARROW KEYS", 279},
  {"to move between options", 280},
  {"and T or INS to mark", 281},
  {"Chmod command", 282},
  {" I can't run the Chmod command on an extfs ", 283},
  {"Set &users", 284},
  {"Set &groups", 285},
  {" Name ", 286},
  {" Owner name ", 287},
  {" Group name ", 288},
  {" Size ", 289},
  {" Chown command ", 290},
  {" User name ", 291},
  {"<Unknown user>", 292},
  {"<Unknown group>", 293},
  {" I can't run the Chown command on an extfs ", 294},
  {" I can't run the Chown command on a tarfs ", 295},
  {" Can not fetch a local copy of %s ", 296},
  {" CD ", 297},
  {"Files tagged, want to cd?", 298},
  {" Filtered view ", 299},
  {" Filter command and arguments:", 300},
  {" Mkdir ", 301},
  {" Enter directory name:", 302},
  {" Filter ", 303},
  {" Set expression for filtering filenames", 304},
  {" Select ", 305},
  {"  Malformed regular expression  ", 306},
  {" Unselect ", 307},
  {"Extension file edit", 308},
  {" Which extension file you want to edit? ", 309},
  {"&User", 310},
  {"&System Wide", 311},
  {"Menu file edit", 312},
  {" Which menu file will you edit? ", 313},
  {"&Local", 314},
  {"&Home", 315},
  {" Compare directories ", 316},
  {" Select compare method: ", 317},
  {"&Quick", 318},
  {"&Size only", 319},
  {"&Thorough", 320},
  {" Both panels should be on the listing view mode to use this command ", 321},
  {" The command history is empty ", 322},
  {" Command history ", 323},
  {"\
 Not an xterm or Linux console; \n\
 the panels cannot be toggled. ", 324},
  {"\
Type `exit' to return to the Midnight Commander\n\
\r\n\
\r", 325},
  {"\
 Couldn't stat %s \n\
 %s ", 326},
  {" Link ", 327},
  {" to:", 328},
  {" link: %s ", 329},
  {" symlink: %s ", 330},
  {" Symlink ", 331},
  {" points to:", 332},
  {" Edit symlink ", 333},
  {" edit symlink: %s ", 334},
  {" Link symbolically ", 335},
  {" Relative symlink ", 336},
  {" relative symlink: %s ", 337},
  {" Enter machine name (F1 for details): ", 338},
  {" Could not chdir to %s ", 339},
  {" Link to a remote machine ", 340},
  {" FTP to machine ", 341},
  {" Socket source routing setup ", 342},
  {" Enter host name to use as a source routing hop: ", 343},
  {" Host name ", 344},
  {" Error while looking up IP address ", 345},
  {" Undelete files on an ext2 file system ", 346},
  {"\
 Enter the file system name where you want to run the\n\
  undelete file system on: (F1 for details)", 347},
  {"Show directory sizes", 348},
  {"Pipe close failed", 349},
  {"Cannot invoke du command.", 350},
  {" Setup saved to ~/", 351},
  {" Setup ", 352},
  {"\
 Can't chdir to '%s' \n\
 %s ", 353},
  {"\
\n\
\n\
\n\
refresh stack underflow!\n\
\n\
\n", 354},
  {"&Unsorted", 355},
  {"&Name", 356},
  {"&Extension", 357},
  {"&Modify time", 358},
  {"&Access time", 359},
  {"&Change time", 360},
  {"&Size", 361},
  {"&Inode", 362},
  {"&Type", 363},
  {"&Links", 364},
  {"N&GID", 365},
  {"N&UID", 366},
  {"&Owner", 367},
  {"&Group", 368},
  {"Unknown", 369},
  {"\
 Can't create temporary command file \n\
 %s ", 370},
  {" Parameter ", 371},
  {" Can't execute commands on a Virtual File System directory ", 372},
  {" file error", 373},
  {"Format of the ", 374},
  {" file error ", 375},
  {"Format of the ~/", 376},
  {"\
 file has changed\n\
with version 3.0. You may want either to\n\
copy it from ", 377},
  {"\
mc.ext or use that\n\
file as an example of how to write it.\n", 378},
  {"mc.ext will be used for this moment.", 379},
  {" Copy ", 380},
  {" Move ", 381},
  {" Delete ", 382},
  {"&Abort", 383},
  {"&Skip", 384},
  {"Source", 385},
  {"Target", 386},
  {"Deleting", 387},
  {"File", 388},
  {"Count", 389},
  {"Bytes", 390},
  {" Invalid target mask ", 391},
  {"\
 Cannot read source link \"%s\" \n\
 %s ", 392},
  {"\
 Cannot make stable symlinks across non-local filesystems: \n\
\n\
 Option Stable Symlinks will be disabled ", 393},
  {"\
 Cannot create target symlink \"%s\" \n\
 %s ", 394},
  {"\
 Cannot overwrite directory \"%s\" \n\
 %s ", 395},
  {"\
 Cannot stat source file \"%s\" \n\
 %s ", 396},
  {" `%s' and `%s' are the same file. ", 397},
  {"\
 Cannot create special file \"%s\" \n\
 %s ", 398},
  {"\
 Cannot chown target file \"%s\" \n\
 %s ", 399},
  {"\
 Cannnot chmod target file \"%s\" \n\
 %s ", 400},
  {"\
 Cannot open source file \"%s\" \n\
 %s ", 401},
  {"\
 Cannot fstat source file \"%s\" \n\
 %s ", 402},
  {"\
 Cannot create target file \"%s\" \n\
 %s ", 403},
  {"\
 Cannot fstat target file \"%s\" \n\
 %s ", 404},
  {"\
 Cannot read source file \"%s\" \n\
 %s ", 405},
  {"\
 Cannot write target file \"%s\" \n\
 %s ", 406},
  {" Can't open target file ", 407},
  {" Can't write to local target file ", 408},
  {"(stalled)", 409},
  {"\
 Cannot close source file \"%s\" \n\
 %s ", 410},
  {"\
 Cannot close target file \"%s\" \n\
 %s ", 411},
  {"\
 Cannot chmod target file \"%s\" \n\
 %s ", 412},
  {"\
 Cannot stat source directory \"%s\" \n\
 %s ", 413},
  {"\
 Source directory \"%s\" is not a directory \n\
 %s ", 414},
  {"\
 Cannot copy cyclic symbolic link \n\
 `%s' ", 415},
  {"\
 Cannot create target directory \"%s\" \n\
 %s ", 416},
  {"\
 Cannot chown target directory \"%s\" \n\
 %s ", 417},
  {"\
 Cannot stat file \"%s\" \n\
 %s ", 418},
  {" `%s' and `%s' are the same file ", 419},
  {" Cannot overwrite directory `%s' ", 420},
  {"\
 Cannot move file \"%s\" to \"%s\" \n\
 %s ", 421},
  {"\
 Cannot remove file \"%s\" \n\
 %s ", 422},
  {" Cannot overwrite directory \"%s\" %s ", 423},
  {" Cannot overwrite file \"%s\" %s ", 424},
  {"\
 Cannot move directory \"%s\" to \"%s\" \n\
 %s ", 425},
  {"\
 Cannot delete file \"%s\" \n\
 %s ", 426},
  {"\
 Cannot remove directory \"%s\" \n\
 %s ", 427},
  {" Internal error: get_file \n", 428},
  {"preserve &Attributes", 429},
  {"follow &Links", 430},
  {"to:", 431},
  {"&Using shell patterns", 432},
  {"&Background", 433},
  {"&Stable Symlinks", 434},
  {"&Dive into subdir if exists", 435},
  {"\
Invalid source pattern `%s' \n\
 %s ", 436},
  {"1Copy", 437},
  {"1Move", 438},
  {"1Delete", 439},
  {"%o %f \"%s\"%m", 440},
  {"%o %d %f%m", 441},
  {"file", 442},
  {"files", 443},
  {"directory", 444},
  {"directories", 445},
  {"files/directories", 446},
  {" with source mask:", 447},
  {" Can't operate on \"..\"! ", 448},
  {" Sorry, I could not put the job in background ", 449},
  {" Internal failure ", 450},
  {" Unknown file operation ", 451},
  {"\
 Destination \"%s\" must be a directory \n\
 %s ", 452},
  {"&Retry", 453},
  {"Target file \"%s\" already exists!", 454},
  {"if &Size differs", 455},
  {"non&E", 456},
  {"&Update", 457},
  {"al&L", 458},
  {"Overwrite all targets?", 459},
  {"&Reget", 460},
  {"ap&Pend", 461},
  {"Overwrite this target?", 462},
  {"Target date: %s, size %d", 463},
  {"Source date: %s, size %d", 464},
  {" File exists ", 465},
  {" Background process: File exists ", 466},
  {"\
\n\
   Directory not empty.   \n\
   Delete it recursively? ", 467},
  {"\
\n\
   Background process: Directory not empty \n\
   Delete it recursively? ", 468},
  {"a&ll", 469},
  {" Type 'yes' if you REALLY want to delete ", 470},
  {"all the directories ", 471},
  {" Recursive Delete ", 472},
  {" Background process: Recursive Delete ", 473},
  {"&Suspend", 474},
  {"&Continue", 475},
  {"&Chdir", 476},
  {"&Again", 477},
  {"&Quit", 478},
  {"Pane&lize", 479},
  {"&View - F3", 480},
  {"&Edit - F4", 481},
  {"Start at:", 482},
  {"Filename:", 483},
  {"Content: ", 484},
  {"&Tree", 485},
  {"Find File", 486},
  {"Grepping in %s", 487},
  {" Find/read ", 488},
  {" Problem reading from child ", 489},
  {"Finished", 490},
  {"Searching %s", 491},
  {"Searching", 492},
  {"Find file", 493},
  {"\
 Help file format error\n\
\004", 494},
  {" Internal bug: Double start of link area ", 495},
  {" Can't find node [Contents] in help file ", 496},
  {" Help ", 497},
  {"\
 Can't open file %s \n\
 %s ", 498},
  {" Can't find node %s in help file ", 499},
  {"Index", 500},
  {"Prev", 501},
  {"&Move", 502},
  {"&Remove", 503},
  {"&Append", 504},
  {"&Insert", 505},
  {"New &Entry", 506},
  {"New &Group", 507},
  {"&Up", 508},
  {"&Add current", 509},
  {"Change &To", 510},
  {"Subgroup - press ENTER to see list", 511},
  {"Active VFS directories", 512},
  {"Directory hotlist", 513},
  {" Directory path ", 514},
  {" Directory label ", 515},
  {"Moving ", 516},
  {"New hotlist entry", 517},
  {"Directory label", 518},
  {"Directory path", 519},
  {" New hotlist group ", 520},
  {"Name of new group", 521},
  {"Label for \"%s\":", 522},
  {" Add to hotlist ", 523},
  {" Remove: ", 524},
  {"\
\n\
 Group not empty.\n\
 Remove it?", 525},
  {" Top level group ", 526},
  {"Hotlist is now kept in file ~/", 527},
  {"MC will load hotlist from ~/", 528},
  {" and then delete [Hotlist] section there", 529},
  {" Hotlist Load ", 530},
  {"MC was unable to write ~/", 531},
  {" file, your old hotlist entries were not deleted", 532},
  {"You have ~/", 533},
  {" file and [Hotlist] section in ~/", 534},
  {"Your ~/", 535},
  {" most probably was created\n", 536},
  {"\
by an earlier development version of MC\n\
and is more actual than ~/", 537},
  {"\
 entries\n\
\n", 538},
  {"\
You can choose between\n\
\n\
  Remove - remove old hotlist entries from ~/", 539},
  {"\
  Keep   - keep your old entries; you will be asked\n\
           the same question next time\n\
  Merge  - add old entries to hotlist as group \"Entries from ~/", 540},
  {"&Keep", 541},
  {"&Merge", 542},
  {" Entries from ~/", 543},
  {" file your old hotlist entries were not deleted", 544},
  {"Midnight Commander %s", 545},
  {"Free nodes %d (%d%%) of %d", 546},
  {"No node information", 547},
  {"Free space ", 548},
  {" (%d%%) of ", 549},
  {"No space information", 550},
  {"Type:      %s ", 551},
  {"non-local vfs", 552},
  {"Device:    %s", 553},
  {"Filesystem: %s", 554},
  {"Accessed:  %s", 555},
  {"Modified:  %s", 556},
  {"Created:   %s", 557},
  {"Size:      ", 558},
  {" (%d blocks)", 559},
  {"Owner:     %s/%s", 560},
  {"Links:     %d", 561},
  {"Mode:      %s (%04o)", 562},
  {"Location:  %Xh:%Xh", 563},
  {"File:       %s", 564},
  {"File:       None", 565},
  {"&Vertical", 566},
  {"&Horizontal", 567},
  {"&Xterm hintbar", 568},
  {"h&Intbar visible", 569},
  {"&Keybar visible", 570},
  {"command &Prompt", 571},
  {"show &Mini status", 572},
  {"menu&Bar visible", 573},
  {"&Equal split", 574},
  {"pe&Rmissions", 575},
  {"&File types", 576},
  {"&Save", 577},
  {" Layout ", 578},
  {" Panel split ", 579},
  {" Highlight... ", 580},
  {" Other options ", 581},
  {"output lines", 582},
  {"Layout", 583},
  {"Other options", 584},
  {"Highlight...", 585},
  {"Panel split", 586},
  {" Learn keys ", 587},
  {" Teach me a key ", 588},
  {"\
Please press the %s\n\
and then wait until this message disappears.\n\
\n\
Then, press it again to see if OK appears\n\
next to its button.\n\
\n\
If you want to escape, press a single Escape key\n\
and wait as well.", 589},
  {" Cannot accept this key ", 590},
  {" You have entered \"%s\"", 591},
  {"OK", 592},
  {"\
It seems that all your keys already\n\
work fine. That's great.", 593},
  {"&Discard", 594},
  {"\
Great! You have a complete terminal database!\n\
All your keys work well.", 595},
  {"Learn keys", 596},
  {"Press all the keys mentioned here. After you have done it, check", 597},
  {"which keys are not marked with OK.  Press space on the missing", 598},
  {"key, or click with the mouse to define it. Move around with Tab.", 599},
  {"\
 The Commander can't change to the directory that \n\
 the subshell claims you are in.  Perhaps you have \n\
 deleted your working directory, or given yourself \n\
 extra access permissions with the \"su\" command? ", 600},
  {"Press any key to continue...", 601},
  {" The shell is already running a command ", 602},
  {" The Midnight Commander ", 603},
  {" Do you really want to quit the Midnight Commander? ", 604},
  {"&Listing mode...", 605},
  {"&Quick view     C-x q", 606},
  {"&Info           C-x i", 607},
  {"&Sort order...", 608},
  {"&Filter...", 609},
  {"&Network link...", 610},
  {"FT&P link...", 611},
  {"&Drive...       M-d", 612},
  {"&Rescan         C-r", 613},
  {"&User menu          F2", 614},
  {"&View               F3", 615},
  {"&Filtered view     M-!", 616},
  {"&Edit               F4", 617},
  {"&Copy               F5", 618},
  {"c&Hmod           C-x c", 619},
  {"&Link            C-x l", 620},
  {"&SymLink         C-x s", 621},
  {"edit s&Ymlink  C-x C-s", 622},
  {"ch&Own           C-x o", 623},
  {"&Advanced chown       ", 624},
  {"&Rename/Move        F6", 625},
  {"&Mkdir              F7", 626},
  {"&Delete             F8", 627},
  {"&Quick cd          M-c", 628},
  {"select &Group      M-+", 629},
  {"u&Nselect group    M-\\", 630},
  {"reverse selec&Tion M-*", 631},
  {"e&Xit              F10", 632},
  {"&Directory tree", 633},
  {"&Find file            M-?", 634},
  {"s&Wap panels          C-u", 635},
  {"switch &Panels on/off C-o", 636},
  {"&Compare directories  C-x d", 637},
  {"e&Xternal panelize    C-x !", 638},
  {"show directory s&Izes", 639},
  {"command &History", 640},
  {"di&Rectory hotlist    C-\\", 641},
  {"&Active VFS list      C-x a", 642},
  {"&Background jobs      C-x j", 643},
  {"&Undelete files (ext2fs only)", 644},
  {"&Listing format edit", 645},
  {"&Extension file edit", 646},
  {"&Menu file edit", 647},
  {"&Configuration...", 648},
  {"c&Onfirmation...", 649},
  {"&Display bits...", 650},
  {"learn &Keys...", 651},
  {"&Virtual FS...", 652},
  {"&Save setup", 653},
  {" Left ", 654},
  {" Above ", 655},
  {" Right ", 656},
  {" Below ", 657},
  {" Information ", 658},
  {"\
 Using the fast reload option may not reflect the exact \n\
 directory contents. In this cases you'll need to do a  \n\
 manual reload of the directory. See the man page for   \n\
 the details.                                           ", 659},
  {"Menu", 660},
  {"Thank you for using GNU Midnight Commander", 661},
  {"with mouse support on xterm%s.\n", 662},
  {" and the Linux console", 663},
  {"The TERM environment variable is unset!\n", 664},
  {"Library directory for the Midnight Commander: %s\n", 665},
  {"Option -m is obsolete. Please look at Display Bits... in the Option's menu\n", 666},
  {"[DEVEL-ONLY: Debug the background code]", 667},
  {"Create command file to set default directory upon exit.", 668},
  {"Force color mode.", 669},
  {"Specify colors (use --help-colors to get a list).", 670},
  {"[DEVEL-ONLY: Debug the subshell.", 671},
  {"Startup the internal editor.", 672},
  {"Shows this help message.", 673},
  {"Help on how to specify colors.", 674},
  {"Log ftpfs commands to the file.", 675},
  {"Prints out the configured paths.", 676},
  {"Force black and white display.", 677},
  {"Disable mouse support.", 678},
  {"Force the concurrent subshell mode", 679},
  {"Disable the concurrent subshell mode.", 680},
  {"Force subshell execution.", 681},
  {"At exit, print the last working directory.", 682},
  {"Reset softkeys (HP terminals only) to their terminfo/termcap default.", 683},
  {"Disables verbose operation (for slow terminals).", 684},
  {"Use simple symbols for line drawing.", 685},
  {"Activate support for the TERMCAP variable.", 686},
  {"Report versionand configuration options.", 687},
  {"Start up into the viewer mode.", 688},
  {"Force xterm mouse support and screen save/restore", 689},
  {"Geometry for the window", 690},
  {"No windows opened at startup", 691},
  {"[this dir] [other dir]", 692},
  {"\
Couldn't open tty line. You have to run mc without the -P flag.\n\
On some systems you may want to run # `which mc`\n", 693},
  {" Notice ", 694},
  {"\
 The Midnight Commander configuration files \n\
 are now stored in the ~/.mc directory, the \n\
 files have been moved now\n", 695},
  {"safe de&Lete", 696},
  {"cd follows lin&Ks", 697},
  {"advanced cho&Wn", 698},
  {"l&Ynx-like motion", 699},
  {"Animation", 700},
  {"ro&Tating dash", 701},
  {"co&Mplete: show all", 702},
  {"&Use internal view", 703},
  {"use internal ed&It", 704},
  {"auto m&Enus", 705},
  {"&Auto save setup", 706},
  {"shell &Patterns", 707},
  {"&Verbose operation", 708},
  {"&Fast dir reload", 709},
  {"mi&X all files", 710},
  {"&Drop down menus", 711},
  {"ma&Rk moves down", 712},
  {"show &Hidden files", 713},
  {"show &Backup files", 714},
  {"&Never", 715},
  {"on dumb &Terminals", 716},
  {"alwa&Ys", 717},
  {" Configure options ", 718},
  {" Panel options ", 719},
  {" Pause after run... ", 720},
  {"Configure options", 721},
  {"Pause after run...", 722},
  {"Panel options", 723},
  {"&Add new", 724},
  {" External panelize ", 725},
  {"External panelize", 726},
  {"Command", 727},
  {"Other command", 728},
  {" Add to external panelize ", 729},
  {" Enter command label: ", 730},
  {" I can't run external panelize while logged on a non local directory ", 731},
  {"Find rejects after patching", 732},
  {"Find *.orig after patching", 733},
  {"Find SUID and SGID programs", 734},
  {"Cannot invoke command.", 735},
  {"Size", 736},
  {"MTime", 737},
  {"Permission", 738},
  {"Perm", 739},
  {"|", 740},
  {"Nl", 741},
  {"GID", 742},
  {"UID", 743},
  {"Owner", 744},
  {"Group", 745},
  {"ATime", 746},
  {"CTime", 747},
  {"Inode", 748},
  {"  %s bytes in %d file%s", 749},
  {"<readlink failed>", 750},
  {"Unknow tag on display format: ", 751},
  {" Do you really want to execute? ", 752},
  {"View", 753},
  {"Edit", 754},
  {"RenMov", 755},
  {"Mkdir", 756},
  {"Couldn't open named pipe %s\n", 757},
  {" The shell is still active. Quit anyway? ", 758},
  {" There are stopped jobs.", 759},
  {" Quit anyway? ", 760},
  {"Warning: Couldn't change to %s.\n", 761},
  {"\
Can't open the %s file for writing:\n\
%s\n", 762},
  {"\
Can't write to the %s file:\n\
%s\n", 763},
  {"Copy \"%s\" directory to:", 764},
  {"Move \"%s\" directory to:", 765},
  {"\
 Can't stat the destination \n\
 %s ", 766},
  {" The destination isn't a directory ", 767},
  {"  Delete %s?  ", 768},
  {"Static", 769},
  {"Dynamc", 770},
  {"\
 Can't chdir to \"%s\" \n\
 %s ", 771},
  {"Rescan", 772},
  {"Forget", 773},
  {"Rmdir", 774},
  {" Invalid shell pattern defination \"%c\". ", 775},
  {" Debug ", 776},
  {" ERROR: ", 777},
  {" True:  ", 778},
  {" False: ", 779},
  {" Warning -- ignoring file ", 780},
  {"\
File %s is not owned by root or you or is world writable.\n\
Using it may compromise your security", 781},
  {" I can't run programs while logged on a non local directory ", 782},
  {" User menu ", 783},
  {"name_trunc: too big", 784},
  {" Pipe failed ", 785},
  {" Dup failed ", 786},
  {"\
File: \n\
\n\
    ", 787},
  {"\
\n\
\n\
has been modified, do you want to save the changes?\n", 788},
  {" Save changes ", 789},
  {" Can't spawn child program ", 790},
  {" Empty output from child filter ", 791},
  {" Could not open file ", 792},
  {" Can't open file \"", 793},
  {"\
 Can't stat file \n\
 ", 794},
  {" Can't view: not a regular file ", 795},
  {" Can not fetch local copy ", 796},
  {"File: %s", 797},
  {"Offset 0x%08x", 798},
  {"Col %d", 799},
  {"%s bytes", 800},
  {"  [grow]", 801},
  {" Search string not found ", 802},
  {" Invalid regular expression ", 803},
  {"\
 The current line number is %d.\n\
 Enter the new line number:", 804},
  {" Enter regexp:", 805},
  {" Enter search string:", 806},
  {"Ascii", 807},
  {"Hex", 808},
  {"Line", 809},
  {"RxSrch", 810},
  {"EdText", 811},
  {"EdHex", 812},
  {"UnWrap", 813},
  {"Wrap", 814},
  {"HxSrch", 815},
  {"Raw", 816},
  {"Parse", 817},
  {"Unform", 818},
  {"Format", 819},
  {"Function key 1", 820},
  {"Function key 2", 821},
  {"Function key 3", 822},
  {"Function key 4", 823},
  {"Function key 5", 824},
  {"Function key 6", 825},
  {"Function key 7", 826},
  {"Function key 8", 827},
  {"Function key 9", 828},
  {"Function key 10", 829},
  {"Function key 11", 830},
  {"Function key 12", 831},
  {"Function key 13", 832},
  {"Function key 14", 833},
  {"Function key 15", 834},
  {"Function key 16", 835},
  {"Function key 17", 836},
  {"Function key 18", 837},
  {"Function key 19", 838},
  {"Function key 20", 839},
  {"Backspace key", 840},
  {"End key", 841},
  {"Up arrow key", 842},
  {"Down arrow key", 843},
  {"Left arrow key", 844},
  {"Right arrow key", 845},
  {"Home key", 846},
  {"Page Down key", 847},
  {"Page Up key", 848},
  {"Insert key", 849},
  {"Delete key", 850},
  {"Completion/M-tab", 851},
  {"+ on keypad", 852},
  {"- on keypad", 853},
  {"* on keypad", 854},
  {"Left arrow keypad", 855},
  {"Right arrow keypad", 856},
  {"Up arrow keypad", 857},
  {"Down arrow keypad", 858},
  {"Home on keypad", 859},
  {"End on keypad", 860},
  {"Page Down keypad", 861},
  {"Page Up keypad", 862},
  {"Insert on keypad", 863},
  {"Delete on keypad", 864},
  {"Enter on keypad", 865},
  {"Slash on keypad", 866},
  {"NumLock on keypad", 867},
  {"Password", 868},
  {"Ok", 869},
  {" There is no other panel to compare contents to ", 870},
  {"GNU Midnight Commander", 871},
  {"The GNOME edition of the Midnight Commander file manager.", 872},
  {"Link", 873},
  {"Do you want to delete ", 874},
  {"Open with...", 875},
  {"Enter extra arguments:", 876},
  {" Could not open %s directory", 877},
  {"New window", 878},
  {"Opens a new window", 879},
  {"Open Terminal", 880},
  {"Opens a terminal", 881},
  {"Copy...", 882},
  {"Copy files", 883},
  {"Rename/Move...", 884},
  {"Rename or move files", 885},
  {"New directory...", 886},
  {"Creates a new folder", 887},
  {"Delete...", 888},
  {"Delete files from disk", 889},
  {"View file", 890},
  {"View raw", 891},
  {"View the file without further processing", 892},
  {"Select group by pattern...", 893},
  {"Selects a group of files", 894},
  {"Unselect group by pattern...", 895},
  {"Un-selects a group of marked files", 896},
  {"Reverse selection", 897},
  {"Reverses the list of tagged files", 898},
  {"Close", 899},
  {"Close this panel", 900},
  {"Exit", 901},
  {"Exit program", 902},
  {"Display mode...", 903},
  {"Set the display mode for the panel", 904},
  {"Sort order...", 905},
  {"Changes the sort order of the files", 906},
  {"Filter...", 907},
  {"Set a filter for the files", 908},
  {"Rescan the directory contents", 909},
  {"Network link...", 910},
  {"Connect to a remote machine", 911},
  {"FTP link...", 912},
  {"Connect to a remote machine with FTP", 913},
  {"Confirmation...", 914},
  {"Confirmation settings", 915},
  {"Options...", 916},
  {"Global option settings", 917},
  {"Virtual FS...", 918},
  {"Virtual File System settings", 919},
  {"Save setup", 920},
  {"Find...", 921},
  {"Locate files on disk", 922},
  {"Hotlist...", 923},
  {"List of favorite sites", 924},
  {"Compare panels...", 925},
  {"Compare panel contents", 926},
  {"External panelize...", 927},
  {"Active VFS list...", 928},
  {"List of active virtual file systems", 929},
  {"Undelete files (ext2fs only)...", 930},
  {"Recover deleted files", 931},
  {"Background jobs...", 932},
  {"List of background operations", 933},
  {"Arrange icons", 934},
  {"Arranges the icons on the desktop", 935},
  {"About", 936},
  {"Information on this program", 937},
  {"Window", 938},
  {"Commands", 939},
  {"Options", 940},
  {"Desktop", 941},
  {"Filename", 942},
  {"Full name: ", 943},
  {"Command:", 944},
  {"Use terminal", 945},
  {"File mode (permissions)", 946},
  {"Current mode: ", 947},
  {"Read", 948},
  {"Write", 949},
  {"Exec", 950},
  {"Special", 951},
  {"User", 952},
  {"Other", 953},
  {"Set UID", 954},
  {"Set GID", 955},
  {"Sticky", 956},
  {"<Unknown>", 957},
  {"General", 958},
  {"Title", 959},
  {"Icon", 960},
  {"Select icon", 961},
  {" Open with...", 962},
  {"Properties", 963},
  {"Open", 964},
  {"Open with", 965},
  {"View unfiltered", 966},
  {"Link...", 967},
  {"Symlink...", 968},
  {"Edit symlink...", 969},
  {"Rename/move..", 970},
  {"All files", 971},
  {"Archives and compressed files", 972},
  {"RPM/DEB files", 973},
  {"Text/Document files", 974},
  {"HTML and SGML files", 975},
  {"Postscript and PDF files", 976},
  {"Spreadsheet files", 977},
  {"Image files", 978},
  {"Video/animation files", 979},
  {"Audio files", 980},
  {"C program files", 981},
  {"C++ program files", 982},
  {"Objective-C program files", 983},
  {"Scheme program files", 984},
  {"Assembler program files", 985},
  {"Misc. program files", 986},
  {"Font files", 987},
  {"Search: ", 988},
  {" %s bytes in %d file%s", 989},
  {"Filter", 990},
  {"Error", 991},
  {"Goto line", 992},
  {"Jump to a specified line number", 993},
  {"Monitor file", 994},
  {"Monitor file growing", 995},
  {"Terminate the viewer", 996},
  {"String search", 997},
  {"Regexp search", 998},
  {"Regular expression search", 999},
  {"Search again...", 1000},
  {"Continue searching", 1001},
  {"Wrap the text", 1002},
  {"Formatted", 1003},
  {"ok", 1004},
  {"cancel", 1005},
  {"help", 1006},
  {"yes", 1007},
  {"no", 1008},
  {"exit", 1009},
  {"abort", 1010},
};

int _msg_tbl_length = 1010;
