set props(.hotlist_new_entry.columns) 4
set props(.hotlist_new_entry.rows)    5
set props(.hotlist_new_entry.frames)  1
set props(.hotlist_new_entry.texts)   0
set props(.hotlist_new_entry.components)   "button-ok button-insert button-append label-pth input-pth label-lbl input-lbl "
set props(.hotlist_new_entry.count)   7
set props(.hotlist_new_entry.props.button-append) "-in .hotlist_new_entry.widgets -column 3 -row 12 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist_new_entry.props.button-insert) "-in .hotlist_new_entry.widgets -column 5 -row 12 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist_new_entry.props.button-cancel) "-in .hotlist_new_entry.widgets -column 7 -row 12 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.hotlist_new_entry.props.label-pth) "-in .hotlist_new_entry.widgets -column 3 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist_new_entry.props.input-pth) "-in .hotlist_new_entry.widgets -column 3 -row 9 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.hotlist_new_entry.props.label-lbl) "-in .hotlist_new_entry.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist_new_entry.props.input-lbl) "-in .hotlist_new_entry.widgets -column 3 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.hotlist_new_entry.frame0) "-in .hotlist_new_entry.widgets -column 1 -row 1 -columnspan 7 -rowspan 11 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.hotlist_new_entry.relief.frame0) groove
