set props(.quick_input_2.columns) 3
set props(.quick_input_2.rows)    5
set props(.quick_input_2.frames)  1
set props(.quick_input_2.texts)   0
set props(.quick_input_2.components)   "button-ok button-cancel label-lbl input-lbl label-pth input-pth "
set props(.quick_input_2.count)   6
set props(.quick_input_2.props.button-ok) "-in .quick_input_2.widgets -column 3 -row 12 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_input_2.props.button-cancel) "-in .quick_input_2.widgets -column 5 -row 12 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.quick_input_2.props.label-lbl) "-in .quick_input_2.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_input_2.props.input-lbl) "-in .quick_input_2.widgets -column 3 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.quick_input_2.props.label-pth) "-in .quick_input_2.widgets -column 3 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_input_2.props.input-pth) "-in .quick_input_2.widgets -column 3 -row 9 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.quick_input_2.frame0) "-in .quick_input_2.widgets -column 1 -row 1 -columnspan 7 -rowspan 11 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.quick_input_2.relief.frame0) groove
set props(.quick_input_2.columns) 3
set props(.quick_input_2.rows)    5
set props(.quick_input_2.frames)  1
set props(.quick_input_2.texts)   0
set props(.quick_input_2.components)   "button-ok button-cancel label-pth input-pth label-lbl input-lbl "
set props(.quick_input_2.count)   6
set props(.quick_input_2.props.button-ok) "-in .quick_input_2.widgets -column 3 -row 12 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_input_2.props.button-cancel) "-in .quick_input_2.widgets -column 5 -row 12 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.quick_input_2.props.label-pth) "-in .quick_input_2.widgets -column 3 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_input_2.props.input-pth) "-in .quick_input_2.widgets -column 3 -row 9 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.quick_input_2.props.label-lbl) "-in .quick_input_2.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_input_2.props.input-lbl) "-in .quick_input_2.widgets -column 3 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.quick_input_2.frame0) "-in .quick_input_2.widgets -column 1 -row 1 -columnspan 7 -rowspan 11 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.quick_input_2.relief.frame0) groove
