/* Definitions for XV_KEY_DATA of different XView objects in Mxc.
   Copyright (C) 1995 Jakub Jelinek.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define KEY_DATA_DLG_HEAD	10000
#define KEY_DATA_PANEL_SPLITX   10001
#define KEY_DATA_AREA_TOP	10002
#define KEY_DATA_AREA_LEFT	10003
#define KEY_DATA_AREA_RIGHT	10004
#define KEY_DATA_AREA_BOTTOM	10005

#define KEY_DATA_TILE		10006
#define KEY_DATA_HSCROLL	10007
#define KEY_DATA_VSCROLL	10008
#define KEY_DATA_PANEL		10009
#define KEY_DATA_PANEL_LASTITEM	10010
#define KEY_DATA_PANEL_MAXX	10011
#define KEY_DATA_PANEL_MAXY	10012
#define KEY_DATA_DIVISION	10013
#define KEY_DATA_BUTTONBAR_IDX	10014
#define KEY_DATA_SELREQ		10015
#define KEY_DATA_COMMAND	10016
#define KEY_DATA_PANEL_CONTAINER 10017
#define KEY_DATA_BOLD_FONT	10018
#define KEY_DATA_PANEL_MINX	10019
#define KEY_DATA_PANEL_MINY	10020
#define KEY_DATA_VIEW		10021
#define KEY_DATA_CANVAS		10022
