set props(.findfile.columns) 5
set props(.findfile.rows)    6 
set props(.findfile.frames)  1
set props(.findfile.texts)   0
set props(.findfile.components)   "content ok tree cancel start label-start label-file label-cont name "
set props(.findfile.count)   9
set props(.findfile.props.content) "-in .findfile.widgets -column 5 -row 7 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.findfile.props.ok) "-in .findfile.widgets -column 3 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.findfile.props.tree) "-in .findfile.widgets -column 5 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.findfile.props.cancel) "-in .findfile.widgets -column 9 -row 13 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.findfile.props.start) "-in .findfile.widgets -column 5 -row 3 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.findfile.props.label-start) "-in .findfile.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.findfile.props.label-file) "-in .findfile.widgets -column 3 -row 5 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.findfile.props.label-cont) "-in .findfile.widgets -column 3 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.findfile.props.name) "-in .findfile.widgets -column 5 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.findfile.frame0) "-in .findfile.widgets -column 1 -row 1 -columnspan 11 -rowspan 9 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.findfile.relief.frame0) groove
