set props(.display.columns) 4
set props(.display.rows)    8
set props(.display.frames)  2
set props(.display.texts)   0
set props(.display.components)   "user-fmt-input mini-status mini-input ok-button cancel-button radio "
set props(.display.count)   6
set props(.display.props.user-fmt-input) "-in .display.widgets -column 5 -row 3 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky sw"
set props(.display.props.mini-status) "-in .display.widgets -column 3 -row 9 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.display.props.mini-input) "-in .display.widgets -column 5 -row 9 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.display.props.ok-button) "-in .display.widgets -column 3 -row 15 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.display.props.cancel-button) "-in .display.widgets -column 7 -row 15 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.display.props.radio) "-in .display.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.display.frame0) "-in .display.widgets -column 1 -row 1 -columnspan 9 -rowspan 5 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.display.relief.frame0) groove
set props(.display.frame1) "-in .display.widgets -column 1 -row 7 -columnspan 9 -rowspan 5 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.display.relief.frame1) groove
