set props(.hotlist_new_group.columns) 4
set props(.hotlist_new_group.rows)    5
set props(.hotlist_new_group.frames)  1
set props(.hotlist_new_group.texts)   0
set props(.hotlist_new_group.components)   "button-append button-insert button-cancel label input "
set props(.hotlist_new_group.count)   5
set props(.hotlist_new_group.props.button-append) "-in .hotlist_new_group.widgets -column 3 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist_new_group.props.button-insert) "-in .hotlist_new_group.widgets -column 5 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist_new_group.props.button-cancel) "-in .hotlist_new_group.widgets -column 7 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.hotlist_new_group.props.label) "-in .hotlist_new_group.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.hotlist_new_group.props.input) "-in .hotlist_new_group.widgets -column 3 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.hotlist_new_group.frame0) "-in .hotlist_new_group.widgets -column 1 -row 1 -columnspan 7 -rowspan 7 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.hotlist_new_group.relief.frame0) groove
