set props(.quick_confirm.columns) 5
set props(.quick_confirm.rows)    7
set props(.quick_confirm.frames)  1
set props(.quick_confirm.texts)   0
set props(.quick_confirm.components)   "ov e o c de "
set props(.quick_confirm.count)   5
set props(.quick_confirm.props.ov) "-in .quick_confirm.widgets -column 5 -row 7 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_confirm.props.e) "-in .quick_confirm.widgets -column 5 -row 9 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_confirm.props.o) "-in .quick_confirm.widgets -column 5 -row 15 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_confirm.props.c) "-in .quick_confirm.widgets -column 9 -row 15 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_confirm.props.de) "-in .quick_confirm.widgets -column 5 -row 5 -columnspan 3 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_confirm.frame0) "-in .quick_confirm.widgets -column 3 -row 3 -columnspan 7 -rowspan 9 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.quick_confirm.relief.frame0) groove
