set props(.quick_vfs.columns) 5
set props(.quick_vfs.rows)    14
set props(.quick_vfs.frames)  3
set props(.quick_vfs.texts)   0
set props(.quick_vfs.components)   "check-ftp-proxy label-sec2 label-tar radio input-limit label-pass input-passwd label-cache input-timeout label-sec input-ftp-proxy button-ok button-cancel label-vfs input-timo-vfs "
set props(.quick_vfs.count)   15
set props(.quick_vfs.props.label-sec2) "-in .quick_vfs.widgets -column 7 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_vfs.props.label-tar) "-in .quick_vfs.widgets -column 3 -row 9 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.quick_vfs.props.radio) "-in .quick_vfs.widgets -column 3 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.quick_vfs.props.input-limit) "-in .quick_vfs.widgets -column 5 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky sw"
set props(.quick_vfs.props.label-pass) "-in .quick_vfs.widgets -column 3 -row 17 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.quick_vfs.props.input-passwd) "-in .quick_vfs.widgets -column 5 -row 17 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_vfs.props.label-cache) "-in .quick_vfs.widgets -column 3 -row 19 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.quick_vfs.props.input-timeout) "-in .quick_vfs.widgets -column 5 -row 19 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_vfs.props.label-sec) "-in .quick_vfs.widgets -column 7 -row 19 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_vfs.props.input-ftp-proxy) "-in .quick_vfs.widgets -column 5 -row 21 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_vfs.props.check-ftp-proxy) "-in .quick_vfs.widgets -column 3 -row 21 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.quick_vfs.props.button-ok) "-in .quick_vfs.widgets -column 3 -row 27 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_vfs.props.button-cancel) "-in .quick_vfs.widgets -column 5 -row 27 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_vfs.props.label-vfs) "-in .quick_vfs.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.quick_vfs.props.input-timo-vfs) "-in .quick_vfs.widgets -column 5 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.quick_vfs.frame0) "-in .quick_vfs.widgets -column 1 -row 1 -columnspan 9 -rowspan 5 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.quick_vfs.relief.frame0) groove
set props(.quick_vfs.frame1) "-in .quick_vfs.widgets -column 1 -row 7 -columnspan 9 -rowspan 7 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.quick_vfs.relief.frame1) groove
set props(.quick_vfs.frame2) "-in .quick_vfs.widgets -column 1 -row 15 -columnspan 9 -rowspan 9 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.quick_vfs.relief.frame2) groove
