set props(.vfshot.columns) 8
set props(.vfshot.rows)    7
set props(.vfshot.frames)  1
set props(.vfshot.texts)   1
set props(.vfshot.components)   "the-lab ct cc listbox "
set props(.vfshot.count)   4
set props(.vfshot.props.the-lab) "-in .vfshot.widgets -column 5 -row 7 -columnspan 9 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.vfshot.props.ct) "-in .vfshot.widgets -column 5 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.vfshot.props.cc) "-in .vfshot.widgets -column 11 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.vfshot.props.listbox) "-in .vfshot.widgets -column 1 -row 1 -columnspan 15 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky {}"
set props(.vfshot.frame0) "-in .vfshot.widgets -column 1 -row 5 -columnspan 15 -rowspan 5 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.vfshot.relief.frame0) groove
set props(.vfshot.text0) "-in .vfshot.widgets -column 3 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky e"
set props(.vfshot.text.text0) "Directory path:"
