set props(.layout.columns) 7
set props(.layout.rows)    7
set props(.layout.frames)  1
set props(.layout.texts)   0
set props(.layout.components)   "m p k v h o s c me "
set props(.layout.count)   9
set props(.layout.props.m) "-in .layout.widgets -column 3 -row 5 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.layout.props.p) "-in .layout.widgets -column 3 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.layout.props.k) "-in .layout.widgets -column 3 -row 9 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.layout.props.v) "-in .layout.widgets -column 3 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.layout.props.h) "-in .layout.widgets -column 3 -row 13 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.layout.props.o) "-in .layout.widgets -column 11 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.layout.props.s) "-in .layout.widgets -column 11 -row 7 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.layout.props.c) "-in .layout.widgets -column 11 -row 11 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky ew"
set props(.layout.props.me) "-in .layout.widgets -column 3 -row 3 -columnspan 1 -rowspan 1 -ipadx 0 -ipady 0 -padx 0 -pady 0 -sticky w"
set props(.layout.frame0) "-in .layout.widgets -column 1 -row 1 -columnspan 5 -rowspan 15 -ipadx 0 -ipady 0 -padx 2 -pady 2 -sticky nesw"
set props(.layout.relief.frame0) groove
